@@ -3,12 +3,11 @@ package Protocol::DBus::WriteMsg;
 use strict;
 use warnings;
 
-use Socket ();
-
-use Protocol::DBus::Socket ();
-
 use parent qw( IO::Framed::Write );
 
+use Socket;
+use Socket::MsgHdr;
+
 my %fh_fds;
 
 sub DESTROY {
@@ -16,13 +15,11 @@ sub DESTROY {
 
     my $fh = delete $fh_fds{ $self->get_write_fh() };
 
-    $self->SUPER::DESTROY() if IO::Framed::Write->can('DESTROY');
-
     return;
 }
 
 sub enqueue_message {
-    my ($self, $buf_sr, $fds_ar, $on_send) = @_;
+    my ($self, $buf_sr, $fds_ar) = @_;
 
     push @{ $fh_fds{$self->get_write_fh()} }, ($fds_ar && @$fds_ar) ? $fds_ar : undef;
 
@@ -33,8 +30,6 @@ sub enqueue_message {
             # We’re done with the message, so we remove the FDs entry,
             # which by here should be undef.
             shift @{ $fh_fds{$self->get_write_fh()} };
-
-            $on_send->() if $on_send;
         },
     );
 
@@ -46,8 +41,6 @@ sub WRITE {
 
     # Only use sendmsg if we actually need to.
     if (my $fds_ar = $fh_fds{ $_[0] }[0]) {
-        die 'Socket::MsgHdr is not loaded!' if !Socket::MsgHdr->can('new');
-
         my $msg = Socket::MsgHdr->new( buf => $_[1] );
 
         $msg->cmsghdr(
@@ -55,7 +48,7 @@ sub WRITE {
             pack( 'I!*', @$fds_ar ),
         );
 
-        my $bytes = Protocol::DBus::Socket::sendmsg_nosignal( $_[0], $msg, 0 );
+        my $bytes = Socket::MsgHdr::sendmsg( $_[0], $msg );
 
         # NOTE: This assumes that, on an incomplete write, the ancillary
         # data (i.e., the FDs) will have been sent, and there is no need
@@ -68,7 +61,7 @@ sub WRITE {
         return $bytes;
     }
 
-    return Protocol::DBus::Socket::send_nosignal( $_[0], $_[1], 0 );
+    goto &IO::Framed::Write::WRITE;
 }
 
 1;
